![Anaplan Connect](img/anaplan-connect-logo.png)

Anaplan Connect is a command-line client and an SDK that makes it easy to communicate with the Anaplan API to execute Import, Export, Delete, and Process actions.
Please read the [NOTICE.txt](NOTICE.txt) and [LICENSE.txt](LICENSE.txt) files before using Anaplan Connect, particularly if you intend to redistribute it.


# Documentation

- The [**Anaplan Connect Guide**](https://help.anaplan.com/en/03144015-44e0-4f28-8cee-13e7ae9d9077-Anaplan-Connect) provides information on installing and using Anaplan Connect from the command-line on Windows, UNIX, GNU/Linux or MacOS environments.
- The [**Anaplan API Guide**](https://help.anaplan.com/en/85bf7966-d357-4f41-a50f-07d51d7b5ac5-Anaplan-API-Guide-and-Reference) provides information useful to developers who want to access the API directly without using Anaplan Connect.
- The [**Anaplan API Reference**](https://anaplanbulkapi20.docs.apiary.io/#) covers all Anaplan APIs.
- The **Javadoc documentation** is useful for Java developers wanting to make use of Anaplan Connect as a library component from their own code. The content is available at ```doc/javadoc```, so open ```doc/javadoc/index.html``` in a browser.


# Developers


## SDK

The SDK provides the ```Service``` class, representing a connection to the Anaplan Connect server, along with associated classes representing accessible resources on the server.
Software using this library should conform to the following pattern:

- A new Service object is instantiated for the service endpoint.
- The Service has many necessary properties set. In particular, service credentials need to be provided.
- The Service object can be used to access Workspace objects, from which Model objects can be accessed, and so on. These are all tied to that Service object.
- When all interaction has been completed, the Service.close() method should be called to release any resources.

The SDK also provides:

- A mechanism to use custom [Transport-providers](src/main/java/com/anaplan/client/transport/README.md) (default: ApacheHttpProvider) for custom API communication.
- A mechanism to use custom [Serialization handlers](src/main/java/com/anaplan/client/serialization/README.md), to help serialize the data to and from Anaplan.


## Build from source

This is a standard Maven project, so to install, test and package do:
```
$ mvn clean install
$ mvn clean test
$ mvn clean package
```

## Deploy to Maven Repository (check with contributers)

To deploy to Maven Github repository at anaplaninc.github.io, do:
```
$ mvn clean deploy
```
Note: Requires a settings.xml file at location ~/.m2/ with contents as shown below and a Github access token. More info at: (https://github.com/settings/tokens):
```
<settings>
  <servers>
    <server>
      <id>github</id>
      <password><GITHUB PERSONAL ACCESS TOKEN></password>
    </server>
  </servers>
</settings>
```

## Update Javadoc Github-pages

To update the Github-pages (branch: gh-pages) site for this repository, with the latest Javadoc, do:
```
$ mvn javadoc:javadoc
$ cp -R target/site/apidocs/* doc/javadoc/.
$ git subtree push --prefix doc origin gh-pages
```


# Releases
## v4.0.0
Features:

- OAuth 2.0 Support in Anaplan Connect.

Bug Fixes:

- Vulnerabilities fixed.

- Illegal Error while exporting a saved view.

- Boolean values from the JDBC connection

## v3.0.0
Features:

- Action-less add, update, upsert and delete operations for list items.

Bug Fixes:

- Unclosed Socket Bug with Http/2.
  
- Data Separator issue for Single Column Files.
  
- Records with new lines skipped when exporting to database.

## v2.1.0
Features:

- Support page dimensions' parameters for Single column and multi column CSV output format.

Bugfixes:

- Improved error reporting for Certificate Authentication.
  
- Valid error message on missing jdbc driver.
  
- Large exports to flat files failing without message.
  
- Visitor (other tenant user) could not run anaplan connect.
  
- Inconsistent CSV format for Single column and Multi column.
  
- False message output for visitor not able to access workspace and model.
  
- Inconsistent Insert statement behavior in JDBC.

## v2.0.0
Features:

- Support names of Workspaces & Models in scripts

- Query list & sizes of workspaces & models

- Query list of modules & views in Anaplan models

- Action-less List item read - Directly read items from model List without requiring to setup Export actions. Currently, supported up to 1M List items.

- Action-less Cell data read - Directly read cells from module views without setting up Export actions. Currently, supported up to 1M cells.

- Version upgrade for mysql connector

Bugfixes:

- Improvements in read time out for open connection 

- Logging improvements for tasks cancellation

- Fixed the encoding issues around Action Names and filenames

- Fixed the Tab column separator issue in Import Data Sources

- Meaningful error message for two delimiters in file definition



## v1.4.4
Features:

- Added Retry mechanism for 5XX errors

- Bugfix- NPE for 0 KB exports

- Bugfix- Cyrillic characters processing around JDBC imports on windows OS.

- Bugfix- JDBC exports for large processing of data

- Removed the usage of loadclass parameter. User scripts will still work even if they provide this parameter.

## v1.4.3

Features:

- Upgrades to public libraries used in Anaplan Connect

- Upgrade to bouncycastle library for direct private key usage

## v1.4.2

Features:

- Export Anaplan data to JDBC targets

- Bug fix: Data corruption issue when uploading multi-byte chars (Shift-JIS, UTF-16LE etc) across chunks

- Bug fix: AC throws null pointer exception with “-VI” command line option

- Bug fix: AC throws exception if Java is installed in “program Files (x86)” Windows directory

- Bug fix: Junk chars in AC terminal logs in Windows

- Bug fix: AC forcing command line parameters for domain/workstation for Basic proxy authentication. Now domain/workstation is required for NTLM auth only

- Bug fix: NULL pointer exception in case of 0 chunks (Anaplan export)


## v1.4.1

Features:

- BugFix to handle NullPointerException when Anaplan returns “Another row has already been processed with this key”
- BugFix - AC did not recognize different column separators
- Allow users to directly use Private key for authentication
- This version will automatically ignore extra text in CA public certificate / Private key. There is extra text appears before the “----BEGIN CERTIFICATE----” part. Users do not need to edit the files to remove these lines.

## v1.4

Features:

- Use of the new and improved Anaplan v2.0 integration API
- Enabling customers to use certificates, obtained from public Certificate Authorities, for Anaplan authentication
- Enabling users to configure retry timeout and number of retries
- Configurable chunk sizes for imports, ranging from 1 to 50 MB
- Improved screen logging, including timestamp, classpath for debug lines, Linux process ID, and more.
- Better security and ease of querying using JDBCparameters moved to “properties” file.

## v1.3.6

Features:

- Mavenized the Anaplan-Connect project, previously on Ant.
- Introduced Log4j.
- Removed cs/ C# code-base for Anaplan-Connect.
- doc/ folder cleanup.
- Fixed unit-tests using mocked API responses.
- Built using JDK 1.7.0_79
- Updated to use Github maven repo at anaplaninc.github.io

## v1.3.5

Bug Fixes as of 19-NOVEMBER-2015

- MOD-753: Anaplan Connect retrieves old or incomplete export file if the export fails to complete
- INTEGRA-699: When a value is blank/missing in the file, Anaplan Connect shifts the data to the left one column.
- BGTRK-10844: Anaplan Connect might hang waiting for task cancellation.
- INTEGRA-246: Remove doc and wadl from connect distribution build.
- RB-68: Change base url for UAT to fix naming convention.

NOTE: Use this for Informatica Connector v1.1.0
